@@ -1,4 +1,9 @@
+from typing import Dict
+
 from django.conf import settings
+from django.contrib.auth import get_user_model
+from django.contrib.postgres.fields import JSONField
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.utils.encoding import force_text
 from django.utils.translation import gettext_lazy as _
@@ -16,6 +21,16 @@ def get_default_scopes():
     return ["openid", "email", "profile"]
 
 
+def get_claim_mapping() -> Dict[str, str]:
+    # Map (some) claim names from https://openid.net/specs/openid-connect-core-1_0.html#Claims
+    # to corresponding field names on the User model
+    return {
+        "email": "email",
+        "first_name": "given_name",
+        "last_name": "family_name",
+    }
+
+
 class OpenIDConnectConfig(SingletonModel):
     """
     Configuration for authentication/authorization via OpenID connect
@@ -53,6 +68,17 @@ class OpenIDConnectConfig(SingletonModel):
         help_text=_("OpenID Connect scopes that are requested during login"),
     )
 
+    oidc_op_discovery_endpoint = models.URLField(
+        _("Discovery endpoint"),
+        max_length=1000,
+        help_text=_(
+            "URL of your OpenID Connect provider discovery endpoint ending with a slash "
+            "(`.well-known/...` will be added automatically). "
+            "If this is provided, the remaining endpoints can be omitted, as "
+            "they will be derived from this endpoint."
+        ),
+        blank=True,
+    )
     oidc_op_jwks_endpoint = models.URLField(
         _("JSON Web Key Set endpoint"),
         max_length=1000,
@@ -61,7 +87,6 @@ class OpenIDConnectConfig(SingletonModel):
         ),
         blank=True,
     )
-
     oidc_op_authorization_endpoint = models.URLField(
         _("Authorization endpoint"),
         max_length=1000,
@@ -86,12 +111,68 @@ class OpenIDConnectConfig(SingletonModel):
         blank=True,
     )
 
+    claim_mapping = JSONField(
+        _("claim mapping"),
+        default=get_claim_mapping,
+        help_text=("Mapping from user-model fields to OIDC claims"),
+    )
+    groups_claim = models.CharField(
+        _("groups claim"),
+        max_length=50,
+        default="roles",
+        help_text=_(
+            "The name of the OIDC claim that holds the values to map to local user groups."
+        ),
+    )
+    sync_groups = models.BooleanField(
+        _("synchronize groups"),
+        default=True,
+        help_text=_(
+            "Synchronize the local user groups with the provided groups. Note that this "
+            "means a user is removed from all groups if there is no group claim. "
+            "Uncheck to manage groups manually."
+        ),
+    )
+    make_users_staff = models.BooleanField(
+        _("make users staff"),
+        default=False,
+        help_text=_(
+            "Users will be flagged as being a staff user automatically. This allows users to login to the admin interface. By default they have no permissions, even if they are staff."
+        ),
+    )
+
     class Meta:
         verbose_name = _("OpenID Connect configuration")
 
     def __str__(self):
         return force_text(self._meta.verbose_name)
 
+    def clean(self):
+        super().clean()
+
+        # validate claim mapping
+        User = get_user_model()
+        for field in self.claim_mapping.keys():
+            try:
+                User._meta.get_field(field)
+            except models.FieldDoesNotExist:
+                raise ValidationError(
+                    {
+                        "claim_mapping": _(
+                            "Field {field} does not exist on the user model"
+                        ).format(field=field)
+                    }
+                )
+
+        if User.USERNAME_FIELD in self.claim_mapping:
+            raise ValidationError(
+                {
+                    "claim_mapping": _(
+                        "The username field may not be in the claim mapping"
+                    ),
+                }
+            )
+
     @property
     def oidc_rp_scopes(self):
         """
