@@ -1,7 +1,7 @@
 # The cache that should be used, e.g. 'default'. Refers to Django CACHES setting.
 # Set to None to disable caching.
-MOZILLA_DJANGO_OIDC_DB_CACHE = None
+MOZILLA_DJANGO_OIDC_DB_CACHE = "oidc"
 
-MOZILLA_DJANGO_OIDC_DB_TIMEOUT = 60 * 5
+MOZILLA_DJANGO_OIDC_DB_CACHE_TIMEOUT = 60 * 5
 
 MOZILLA_DJANGO_OIDC_DB_PREFIX = "oidc"
